"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generatePaginatorArgs = exports.generateBooleanField = exports.generateEnumField = exports.generateJoinableField = exports.generateCreatedByField = exports.generateIdField = exports.generateUpdatedAtField = exports.generateCreatedAtField = void 0;
const sequelize_1 = require("sequelize");
const dataType_1 = require("./dataType");
function generateCreatedAtField() {
    return {
        created_at: {
            type: dataType_1.dataTypes.DATETIME,
            mysqlOptions: {
                type: sequelize_1.DataTypes.DATE,
                allowNull: false,
                defaultValue: sequelize_1.Sequelize.literal('CURRENT_TIMESTAMP'),
                getter: (field) => "UNIX_TIMESTAMP(" + field + ")",
            },
        }
    };
}
exports.generateCreatedAtField = generateCreatedAtField;
;
function generateUpdatedAtField() {
    return {
        updated_at: {
            type: dataType_1.dataTypes.DATETIME,
            allowNull: true,
            mysqlOptions: {
                type: sequelize_1.DataTypes.DATE,
                allowNull: true,
                getter: (field) => "UNIX_TIMESTAMP(" + field + ")",
                setter: () => "CURRENT_TIMESTAMP"
            },
        }
    };
}
exports.generateUpdatedAtField = generateUpdatedAtField;
;
function generateIdField() {
    return {
        id: {
            type: dataType_1.dataTypes.ID,
            mysqlOptions: {
                type: sequelize_1.DataTypes.INTEGER,
                autoIncrement: true,
                primaryKey: true,
                allowNull: false,
            },
            filterable: true
        },
    };
}
exports.generateIdField = generateIdField;
;
function generateCreatedByField(service) {
    return generateJoinableField({ name: "created_by", service });
}
exports.generateCreatedByField = generateCreatedByField;
;
function generateJoinableField(args) {
    const { name, service, options, mysqlOptions, required = true } = args;
    return {
        [name !== null && name !== void 0 ? name : service.__typename]: Object.assign(Object.assign({ type: service.__typename, mysqlOptions: Object.assign(Object.assign({ type: sequelize_1.DataTypes.INTEGER, joinInfo: {
                    type: service.__typename,
                } }, mysqlOptions), { allowNull: !required }), addable: true, filterable: true }, options), { allowNull: !required }),
    };
}
exports.generateJoinableField = generateJoinableField;
;
function generateEnumField(name, service, options, mysqlOptions) {
    return {
        [name]: Object.assign({ type: service.__typename, mysqlOptions: Object.assign({ type: sequelize_1.DataTypes.INTEGER, allowNull: false }, mysqlOptions), resolver: async (context, req, currentObject, query, args, parent) => {
                return service.getRecord(req, {
                    id: currentObject[name]
                }, query);
            }, transform: {
                setter: (value) => {
                    return service.enum[value];
                }
            }, filterable: true, updateable: true, addable: true }, options),
    };
}
exports.generateEnumField = generateEnumField;
;
function generateBooleanField(name, options, mysqlOptions) {
    return {
        [name]: Object.assign({ type: dataType_1.dataTypes.BOOLEAN, mysqlOptions: Object.assign({ type: sequelize_1.DataTypes.BOOLEAN, defaultValue: true, allowNull: false }, mysqlOptions), transform: {
                getter: (value) => value === 1
            }, addable: true, updateable: true }, options),
    };
}
exports.generateBooleanField = generateBooleanField;
;
function generatePaginatorArgs(service, excludeProperties = []) {
    const getArgs = {
        id: { type: dataType_1.dataTypes.ID },
        first: { type: dataType_1.dataTypes.INTEGER },
        after: { type: dataType_1.dataTypes.ID },
        search: { type: dataType_1.dataTypes.STRING },
        sortBy: { type: [dataType_1.dataTypes.STRING] },
        sortDesc: { type: [dataType_1.dataTypes.BOOLEAN] },
    };
    if (service.filterFieldsMap) {
        for (const field in service.filterFieldsMap) {
            if (!excludeProperties.includes(field)) {
                getArgs[field] = { type: dataType_1.dataTypes.STRING };
            }
        }
    }
    return getArgs;
}
exports.generatePaginatorArgs = generatePaginatorArgs;
;
//# sourceMappingURL=typeDef.js.map