"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRootResolvers = void 0;
const dataType_1 = require("../tier0/dataType");
const typeDef_1 = require("../tier0/typeDef");
function generateRootResolvers(rootResolvers, service, typeDefs, params) {
    const capitalizedClass = service.__typename.charAt(0).toUpperCase() + service.__typename.slice(1);
    params.methods.forEach((method) => {
        const capitalizedMethod = method.charAt(0).toUpperCase() + method.slice(1);
        switch (method) {
            case "get":
                rootResolvers.query[method + capitalizedClass] = {
                    method: "get",
                    route: "/" + service.__typename + "/:id",
                    type: service.__typename,
                    args: typeDef_1.generatePaginatorArgs(service),
                    resolver: (req) => {
                        var _a;
                        return service.getRecord(req, Object.assign(Object.assign(Object.assign({}, req.query), req.params.id !== ":id" && { id: req.params.id }), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "getMultiple":
                rootResolvers.query[method + capitalizedClass] = {
                    method: "get",
                    route: "/" + service.__typename,
                    type: service.paginator.__typename,
                    args: typeDef_1.generatePaginatorArgs(service),
                    resolver: (req) => {
                        var _a;
                        return service.paginator.getRecord(req, Object.assign(Object.assign(Object.assign({}, req.query), req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "delete":
                rootResolvers.mutation[method + capitalizedClass] = {
                    method: "delete",
                    route: "/" + service.__typename + "/:id",
                    type: service.__typename,
                    args: {
                        id: { type: dataType_1.dataTypes.ID, required: true }
                    },
                    resolver: (req) => {
                        var _a;
                        return service.deleteRecord(req, Object.assign(Object.assign({}, req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "update":
                const updateArgs = {};
                for (const field in typeDefs[service.__typename]) {
                    if (typeDefs[service.__typename][field].updateable) {
                        updateArgs[field] = { type: typeDefs[service.__typename][field].type };
                    }
                }
                rootResolvers.mutation[method + capitalizedClass] = {
                    method: "put",
                    route: "/" + service.__typename + "/:id",
                    type: service.__typename,
                    args: Object.assign({ id: { type: dataType_1.dataTypes.ID, required: true } }, updateArgs),
                    resolver: (req) => {
                        var _a;
                        return service.updateRecord(req, Object.assign(Object.assign(Object.assign({}, req.body), req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "create":
                const createArgs = {};
                for (const field in typeDefs[service.__typename]) {
                    if (typeDefs[service.__typename][field].addable) {
                        createArgs[field] = { type: typeDefs[service.__typename][field].type };
                    }
                }
                rootResolvers.mutation[method + capitalizedClass] = {
                    method: "post",
                    route: "/" + service.__typename,
                    type: service.__typename,
                    args: createArgs,
                    resolver: (req) => {
                        var _a;
                        return service.createRecord(req, Object.assign(Object.assign(Object.assign({}, req.body), req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "created":
                rootResolvers.subscription[service.__typename + capitalizedMethod] = {
                    method: "post",
                    route: "/subscribe/" + service.__typename + capitalizedMethod,
                    type: service.__typename,
                    resolver: (req) => {
                        var _a;
                        return service.subscribeToMultipleItem(service.__typename + capitalizedMethod, req, Object.assign(Object.assign({}, req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "deleted":
                rootResolvers.subscription[service.__typename + capitalizedMethod] = {
                    method: "post",
                    route: "/subscribe/" + service.__typename + capitalizedMethod,
                    type: service.__typename,
                    resolver: (req) => {
                        var _a;
                        return service.subscribeToSingleItem(service.__typename + capitalizedMethod, req, Object.assign(Object.assign({}, req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
            case "updated":
                rootResolvers.subscription[service.__typename + capitalizedMethod] = {
                    method: "post",
                    route: "/subscribe/" + service.__typename + capitalizedMethod,
                    type: service.__typename,
                    resolver: (req) => {
                        var _a;
                        return service.subscribeToSingleItem(service.__typename + capitalizedMethod, req, Object.assign(Object.assign({}, req.params), (_a = req.jql) === null || _a === void 0 ? void 0 : _a.__args), req.jql);
                    }
                };
                break;
        }
    });
}
exports.generateRootResolvers = generateRootResolvers;
;
//# sourceMappingURL=rootResolver.js.map