"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generators = exports.typeDefHelper = exports.dataTypes = exports.rootResolverHelper = exports.resolverHelper = exports.mysqlHelper = exports.serviceHelper = exports.subscriptionHelper = exports.getTypeDefs = exports.getSchema = exports.initialize = void 0;
const router_1 = require("./helpers/tier1/router");
const schema_1 = require("./helpers/tier0/schema");
const subscription_1 = require("./helpers/tier2/subscription");
const pusher_1 = require("./utils/pusher");
// utils
const mysql2_1 = require("./utils/mysql2");
let exportedSchema;
function initialize(app, schema, params = {}) {
    const { mysqlEnv, pusherEnv, debug, allowedOrigins } = params;
    exportedSchema = schema;
    mysql2_1.default.initializePool(mysqlEnv, debug);
    pusher_1.initializePusher(pusherEnv);
    app.use((req, res, next) => {
        // aggregate all root resolvers
        const allRootResolvers = {};
        for (const resolverType in schema.rootResolvers) {
            for (const prop in schema.rootResolvers[resolverType]) {
                allRootResolvers[prop] = schema.rootResolvers[resolverType][prop];
            }
        }
        // handle jql queries
        if (req.method === "POST" && req.url === "/jql") {
            if (req.body.action in allRootResolvers) {
                // map from action to method + url
                req.method = allRootResolvers[req.body.action].method;
                req.url = allRootResolvers[req.body.action].route;
                //add the app route that we are going to use
                app[allRootResolvers[req.body.action].method](allRootResolvers[req.body.action].route, router_1.default.externalFnWrapper(allRootResolvers[req.body.action].resolver));
            }
            req.jql = req.body.query || {};
        }
        else {
            //if not using jql, must populate all the routes
            for (const prop in allRootResolvers) {
                app[allRootResolvers[prop].method](allRootResolvers[prop].route, router_1.default.externalFnWrapper(allRootResolvers[prop].resolver));
            }
        }
        next();
    });
    app.set('json replacer', function (key, value) {
        // undefined values are set to `null`
        if (typeof value === "undefined") {
            return null;
        }
        return value;
    });
    app.use(function (req, res, next) {
        const origin = (Array.isArray(allowedOrigins) && allowedOrigins.length) ? (allowedOrigins.includes(req.headers.origin) ? req.headers.origin : allowedOrigins[0]) : "*";
        res.header("Access-Control-Allow-Origin", origin);
        if (origin !== "*") {
            res.header("Vary", "Origin");
        }
        res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept, Authorization, Cache-Control");
        res.header('Access-Control-Allow-Methods', 'PUT, POST, GET, DELETE, OPTIONS');
        next();
    });
    app.options('*', function (req, res, next) {
        res.header('Access-Control-Max-Age', "86400");
        res.sendStatus(200);
    });
    app.get("/schema", function (req, res) {
        res.send(schema_1.generateSchema(schema));
    });
    app.get("/graphqlschema", function (req, res) {
        res.send(schema_1.generateGraphqlSchema(schema));
    });
    app.post('/pusher/auth', subscription_1.handlePusherAuth);
    app.post('/pusher/webhook', subscription_1.handleWebhook);
    app.post('/mysql/sync', function (req, res) {
        var _a;
        //loop through typeDefs to identify needed mysql tables
        mysql2_1.default.initializeSequelize(mysqlEnv);
        const sequelize = mysql2_1.default.getSequelizeInstance();
        for (const type in schema.typeDefs) {
            const definition = {};
            let properties = 0;
            for (const prop in schema.typeDefs[type]) {
                if (prop !== 'id' && ((_a = schema.typeDefs[type][prop].mysqlOptions) === null || _a === void 0 ? void 0 : _a.type)) {
                    definition[prop] = schema.typeDefs[type][prop].mysqlOptions;
                    properties++;
                }
            }
            if (properties > 0) {
                sequelize.define(type, definition, { timestamps: false, freezeTableName: true });
            }
        }
        //define the jql subscription table
        sequelize.define('jqlSubscription', subscription_1.typeDef, { timestamps: false, freezeTableName: true });
        sequelize.sync({ alter: true }).then(() => {
            console.log("Drop and re-sync db.");
            sequelize.close();
            res.send({});
        });
    });
}
exports.initialize = initialize;
;
exports.getSchema = () => exportedSchema;
exports.getTypeDefs = () => exportedSchema.typeDefs;
exports.subscriptionHelper = require("./helpers/tier2/subscription");
exports.serviceHelper = require("./services/service");
exports.mysqlHelper = require("./helpers/tier1/mysql");
exports.resolverHelper = require("./resolvers/resolver");
exports.rootResolverHelper = require("./helpers/tier2/rootResolver");
var dataType_1 = require("./helpers/tier0/dataType");
Object.defineProperty(exports, "dataTypes", { enumerable: true, get: function () { return dataType_1.dataTypes; } });
exports.typeDefHelper = require("./helpers/tier0/typeDef");
exports.generators = require("./services/generators");
//# sourceMappingURL=index.js.map