"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.countTableRows = exports.resolveTableRows = exports.deleteTableRow = exports.updateTableRow = exports.addTableRow = void 0;
const jql_1 = require("../helpers/tier0/jql");
const shared_1 = require("../helpers/tier0/shared");
const __1 = require("..");
//validates the add fields, and then does the add operation
async function addTableRow(classname, args, adminFields = {}, ignore = false) {
    var _a, _b, _c, _d;
    //resolve the setters
    const validQuery = __1.getTypeDefs()[classname];
    //assemble the mysql fields
    const mysqlFields = {};
    //handle the custom setters
    const customResolvers = {};
    for (const field in args) {
        if (field in validQuery) {
            if (validQuery[field].addable) {
                //if there's a setter to transform the input, use that
                mysqlFields[field] = ((_a = validQuery[field].transform) === null || _a === void 0 ? void 0 : _a.setter) ? await ((_b = validQuery[field].transform) === null || _b === void 0 ? void 0 : _b.setter(args[field])) : args[field];
            }
            else if (validQuery[field].setter) {
                customResolvers[field] = validQuery[field].setter;
            }
        }
    }
    //process adminFields
    for (const field in adminFields) {
        if (field in validQuery) {
            if (validQuery[field].setter) {
                customResolvers[field] = validQuery[field].setter;
            }
            else {
                mysqlFields[field] = ((_c = validQuery[field].transform) === null || _c === void 0 ? void 0 : _c.setter) ? await ((_d = validQuery[field].transform) === null || _d === void 0 ? void 0 : _d.setter(adminFields[field])) : adminFields[field];
            }
        }
    }
    //do the mysql first
    const addResults = await __1.mysqlHelper.insertTableRow(classname, mysqlFields, ignore);
    const resultObject = {
        id: addResults.insertId
    };
    //handle the custom setter functions, which might rely on primary keys
    for (const field in customResolvers) {
        await customResolvers[field](classname, args[field], resultObject);
    }
    return resultObject;
}
exports.addTableRow = addTableRow;
//validates the add fields, and then does the add operation
async function updateTableRow(classname, args, adminArgs = {}, whereArray) {
    var _a, _b, _c, _d;
    //resolve the setters
    const validQuery = __1.getTypeDefs()[classname];
    //assemble the mysql fields
    const mysqlFields = {};
    //handle the custom setters
    const customResolvers = {};
    for (const field in args) {
        if (field in validQuery) {
            if (validQuery[field].updateable) {
                //if there's a setter to transform the input, use that
                mysqlFields[field] = ((_a = validQuery[field].transform) === null || _a === void 0 ? void 0 : _a.setter) ? await ((_b = validQuery[field].transform) === null || _b === void 0 ? void 0 : _b.setter(args[field])) : args[field];
            }
            else if (validQuery[field].updater) {
                customResolvers[field] = validQuery[field].updater;
            }
        }
    }
    //process adminFields
    for (const field in adminArgs) {
        if (field in validQuery) {
            if (validQuery[field].updateable) {
                //if there's a setter to transform the input, use that
                mysqlFields[field] = ((_c = validQuery[field].transform) === null || _c === void 0 ? void 0 : _c.setter) ? await ((_d = validQuery[field].transform) === null || _d === void 0 ? void 0 : _d.setter(adminArgs[field])) : adminArgs[field];
            }
            else if (validQuery[field].updater) {
                customResolvers[field] = validQuery[field].updater;
            }
        }
    }
    //do the mysql first
    await __1.mysqlHelper.updateTableRow(classname, mysqlFields, {}, whereArray);
    const resultObject = {
        id: args.id
    };
    //handle the custom setter functions, which might rely on primary keys
    for (const field in customResolvers) {
        await customResolvers[field](classname, args[field], resultObject);
    }
    return resultObject;
}
exports.updateTableRow = updateTableRow;
//performs the delete operation
async function deleteTableRow(classname, args, whereArray) {
    //resolve the deleters
    const validQuery = __1.getTypeDefs()[classname];
    //handle the custom deleters
    const customResolvers = {};
    for (const field in validQuery) {
        if (validQuery[field].deleter) {
            customResolvers[field] = validQuery[field].deleter;
        }
    }
    //do the mysql first
    await __1.mysqlHelper.removeTableRow(classname, whereArray);
    const resultObject = {
        id: args.id
    };
    //handle the custom deleter functions, which might rely on primary keys
    for (const field in customResolvers) {
        await customResolvers[field](classname, null, resultObject);
    }
    return resultObject;
}
exports.deleteTableRow = deleteTableRow;
async function resolveTableRows(classname, context, req, jqlQuery, args = {}) {
    const validQuery = __1.getTypeDefs()[classname];
    //validate graphql
    const validatedGraphql = jql_1.default.validateJsonqlQuery(jqlQuery.select, classname);
    jqlQuery.select = validatedGraphql.validatedQuery;
    let hasMysqlFields = false;
    //handle mysql fields - if any
    for (const prop in jqlQuery.select) {
        if (!jqlQuery.select[prop].resolver) {
            hasMysqlFields = true;
            break;
        }
    }
    //validate where fields and remove any that are not filterable
    if (Array.isArray(jqlQuery.where)) {
        jqlQuery.where.forEach(ele => {
            for (const field in ele) {
                if (field in validQuery) {
                    if (!validQuery[field].filterable) {
                        delete ele[field];
                    }
                }
            }
        });
    }
    const returnArray = hasMysqlFields ? shared_1.default.collapseObjectArray(await __1.mysqlHelper.fetchTableRows(classname, jqlQuery)) : [{ __typename: context.__typename }];
    //apply transformations of results
    for (const returnObject of returnArray) {
        await jql_1.default.handleTransformQueries(returnObject, validatedGraphql.validatedQuery, context, req, args);
    }
    //handle resolved fields
    for (const returnObject of returnArray) {
        await jql_1.default.handleResolvedQueries(returnObject, validatedGraphql.validatedResolvedQuery, context, req, args);
    }
    //handle aggregated fields
    await jql_1.default.handleAggregatedQueries(returnArray, validatedGraphql.validatedAggregatedQuery, context, req, args);
    return returnArray;
}
exports.resolveTableRows = resolveTableRows;
async function countTableRows(classname, filterArray) {
    const validQuery = __1.getTypeDefs()[classname];
    //validate where fields and remove any that are not filterable
    if (Array.isArray(filterArray)) {
        filterArray.forEach(ele => {
            for (const field in ele) {
                if (field in validQuery) {
                    if (!validQuery[field].filterable) {
                        delete ele[field];
                    }
                }
            }
        });
    }
    return __1.mysqlHelper.countTableRows(classname, filterArray);
}
exports.countTableRows = countTableRows;
//# sourceMappingURL=resolver.js.map